@@ -20,6 +20,7 @@ from .code_mappings import (
     RoborockFanPowerCode,
     RoborockFanSpeedP10,
     RoborockFanSpeedQ7Max,
+    RoborockFanSpeedQRevoCurv,
     RoborockFanSpeedQRevoMaster,
     RoborockFanSpeedS6Pure,
     RoborockFanSpeedS7,
@@ -30,12 +31,15 @@ from .code_mappings import (
     RoborockMopIntensityCode,
     RoborockMopIntensityP10,
     RoborockMopIntensityQ7Max,
+    RoborockMopIntensityQRevoCurv,
     RoborockMopIntensityQRevoMaster,
     RoborockMopIntensityS5Max,
     RoborockMopIntensityS6MaxV,
     RoborockMopIntensityS7,
     RoborockMopIntensityS8MaxVUltra,
     RoborockMopModeCode,
+    RoborockMopModeQRevoCurv,
+    RoborockMopModeQRevoMaster,
     RoborockMopModeS7,
     RoborockMopModeS8MaxVUltra,
     RoborockMopModeS8ProUltra,
@@ -51,6 +55,7 @@ from .const import (
     ROBOROCK_G10S_PRO,
     ROBOROCK_P10,
     ROBOROCK_Q7_MAX,
+    ROBOROCK_QREVO_CURV,
     ROBOROCK_QREVO_MASTER,
     ROBOROCK_QREVO_MAXV,
     ROBOROCK_QREVO_PRO,
@@ -68,6 +73,7 @@ from .const import (
     SENSOR_DIRTY_REPLACE_TIME,
     SIDE_BRUSH_REPLACE_TIME,
     STRAINER_REPLACE_TIME,
+    ROBOROCK_G20S_Ultra,
 )
 from .exceptions import RoborockException
 
@@ -221,6 +227,7 @@ class RRiot(RoborockBase):
 
 @dataclass
 class UserData(RoborockBase):
+    rriot: RRiot
     uid: int | None = None
     tokentype: str | None = None
     token: str | None = None
@@ -229,7 +236,6 @@ class UserData(RoborockBase):
     countrycode: str | None = None
     country: str | None = None
     nickname: str | None = None
-    rriot: RRiot | None = None
     tuya_device_state: int | None = None
     avatarurl: str | None = None
 
@@ -434,6 +440,12 @@ class HomeDataRoom(RoborockBase):
     name: str
 
 
+@dataclass
+class HomeDataScene(RoborockBase):
+    id: int
+    name: str
+
+
 @dataclass
 class HomeData(RoborockBase):
     id: int
@@ -577,7 +589,14 @@ class Q7MaxStatus(Status):
 class QRevoMasterStatus(Status):
     fan_power: RoborockFanSpeedQRevoMaster | None = None
     water_box_mode: RoborockMopIntensityQRevoMaster | None = None
-    mop_mode: RoborockMopModeS7 | None = None
+    mop_mode: RoborockMopModeQRevoMaster | None = None
+
+
+@dataclass
+class QRevoCurvStatus(Status):
+    fan_power: RoborockFanSpeedQRevoCurv | None = None
+    water_box_mode: RoborockMopIntensityQRevoCurv | None = None
+    mop_mode: RoborockMopModeQRevoCurv | None = None
 
 
 @dataclass
@@ -638,6 +657,7 @@ ModelStatus: dict[str, type[Status]] = {
     ROBOROCK_S5_MAX: S5MaxStatus,
     ROBOROCK_Q7_MAX: Q7MaxStatus,
     ROBOROCK_QREVO_MASTER: QRevoMasterStatus,
+    ROBOROCK_QREVO_CURV: QRevoCurvStatus,
     ROBOROCK_S6: S6PureStatus,
     ROBOROCK_S6_MAXV: S6MaxVStatus,
     ROBOROCK_S6_PURE: S6PureStatus,
@@ -646,6 +666,7 @@ ModelStatus: dict[str, type[Status]] = {
     ROBOROCK_S8: S8Status,
     ROBOROCK_S8_PRO_ULTRA: S8ProUltraStatus,
     ROBOROCK_G10S_PRO: S7MaxVStatus,
+    ROBOROCK_G20S_Ultra: QRevoMasterStatus,
     ROBOROCK_P10: P10Status,
     # These likely are not correct,
     # but i am currently unable to do my typical reverse engineering/ get any data from users on this,
@@ -880,26 +901,28 @@ class RoborockProductSpec(RoborockBase):
 
 @dataclass
 class RoborockProduct(RoborockBase):
-    id: int
-    name: str
-    model: str
-    packagename: str
-    ssid: str
-    picurl: str
-    cardpicurl: str
-    medium_cardpicurl: str
-    resetwifipicurl: str
-    resetwifitext: dict
-    tuyaid: str
-    status: int
-    rriotid: str
-    cardspec: str
-    pictures: list
-    nc_mode: str
-    scope: None
-    product_tags: list
-    agreements: list
-    plugin_pic_url: None
+    id: int | None = None
+    name: str | None = None
+    model: str | None = None
+    packagename: str | None = None
+    ssid: str | None = None
+    picurl: str | None = None
+    cardpicurl: str | None = None
+    mediumCardpicurl: str | None = None
+    resetwifipicurl: str | None = None
+    configPicUrl: str | None = None
+    pluginPicUrl: str | None = None
+    resetwifitext: dict | None = None
+    tuyaid: str | None = None
+    status: int | None = None
+    rriotid: str | None = None
+    pictures: list | None = None
+    ncMode: str | None = None
+    scope: str | None = None
+    product_tags: list | None = None
+    agreements: list | None = None
+    cardspec: str | None = None
+    plugin_pic_url: str | None = None
     products_specification: RoborockProductSpec | None = None
 
     def __post_init__(self):
